from django.test import TestCase

from template_engines.templatetags.utils import size_parser, resize

from ..settings import IMAGE_PATH


class TestUtils(TestCase):

    def test_size_parser_no_unit(self):
        self.assertEqual(size_parser(1000), 1000.0)
        self.assertEqual(size_parser('1000'), 1000.0)

    def test_size_parser_dxa(self):
        self.assertEqual(size_parser('1000dxa'), 1000.0)

    def test_size_parser_pt_to_dxa(self):
        self.assertEqual(size_parser('1000pt'), 20000.0)

    def test_size_parser_px_to_dxa(self):
        self.assertEqual(size_parser('1000px'), 15000.0)

    def test_size_parser_in_to_dxa(self):
        self.assertEqual(size_parser('72in'), 103680.0)

    def test_size_parser_cm_to_dxa(self):
        self.assertEqual(size_parser('1cm'), 566.9291338582677)
        self.assertEqual(size_parser('1.5cm'), 850.3937007874015)

    def test_size_parser_emu_to_dxa(self):
        self.assertEqual(size_parser('635emu'), 1.0)

    def test_resize_width_height(self):
        width, height = resize(b'', '100pt', '100pt')
        self.assertEqual(width, 2000.0)
        self.assertEqual(height, 2000.0)
        width, height = resize(b'', '100pt', '100pt', odt=False)
        self.assertEqual(width, 1270000.0)
        self.assertEqual(height, 1270000.0)

    def test_resize_image(self):
        img = open(IMAGE_PATH, 'rb').read()
        width, height = resize(img, None, None)
        self.assertEqual(width, 16697.0)
        self.assertEqual(height, 5763.431472081218)



from base64 import b64encode

from django import template
from django.utils.safestring import mark_safe

from .utils import resize

register = template.Library()
ODT_IMAGE = (
    '<draw:frame draw:name="img1" svg:width="{0}" svg:height="{1}">'
    + '<draw:image xlink:type="simple" xlink:show="embed" xlink:actuate="onLoad">'
    + '<office:binary-data>{2}</office:binary-data>'
    + '</draw:image></draw:frame>'
)


@register.simple_tag
def odt_image_loader(image):
    """
    Replace a tag by an image you specified.
    You must add an entry to the ``context`` var that is a dict with at least a ``content`` key
    whose value is a byte object. You can also specify ``width`` and ``height``, otherwise it will
    automatically resize your image.
    """
    width = image.get('width')
    height = image.get('height')
    content = image.get('content')

    width, height = resize(content, width, height)

    return mark_safe(ODT_IMAGE.format(width, height, b64encode(content).decode()))  # nosec



"""
Contains all generic functions that can be used to build backends.
"""
import re
from tempfile import NamedTemporaryFile
from zipfile import ZipFile

DOCX_RELATIONSHIP = (
    '<Relationship Id="{0}" Type="http://schemas.openxmlformats.org/officeDocument/2006/'
    + 'relationships/image" Target="media/{0}"/></Relationships>')


def modify_libreoffice_doc(file_path, xml_path, rendered):
    """
    Modify a libreoffice document (docx and odt only for the moment).

    :param file_path: the path of the zip file.
    :type file_path: str

    :param xml_path: path to the xml representation of the document.
    :type xml_path: str

    :param rendered: content to put in the xml representation of the document.
    :type rendered: str

    :returns: the modified file as a byte object.
    """
    temp_file = NamedTemporaryFile()
    with ZipFile(file_path, 'r') as read_zip_file:
        info_list = read_zip_file.infolist()
        with ZipFile(temp_file.name, 'w') as write_zip_file:
            for item in info_list:
                if item.filename == xml_path:
                    write_zip_file.writestr(item, rendered)
                else:
                    write_zip_file.writestr(item, read_zip_file.read(item.filename))

    with open(temp_file.name, 'rb') as read_file:
        return read_file.read()


def add_image_in_docx_template(bfile, image):
    """
    Makes an image available for a docx document.

    :param bfile: the docx document.
    :type bfile: bytes

    :param image: a dictionary with at least two entries: the image's name and the image's content.
    :type image: dict
    """
    temp_file = NamedTemporaryFile()
    with open(temp_file.name, 'wb') as writer:
        writer.write(bfile)

    temp_doc = NamedTemporaryFile()
    with ZipFile(temp_file.name, 'r') as read_zip_file:
        with ZipFile(temp_doc.name, 'w') as write_zip_file:
            # Add a relationship tag to word/_rels/document.xml.rels
            for item in read_zip_file.infolist():
                if item.filename == 'word/_rels/document.xml.rels':
                    doc_relationships = read_zip_file.read(item.filename).decode()
                    doc_relationships = re.sub(
                        '</Relationships>',
                        DOCX_RELATIONSHIP.format(image.get('name')),
                        doc_relationships)
                    write_zip_file.writestr(item.filename, doc_relationships)
                else:
                    write_zip_file.writestr(item, read_zip_file.read(item.filename))

            # Add the image in the word/media folder
            temp_image = NamedTemporaryFile()
            with open(temp_image.name, 'wb') as writer:
                writer.write(image.get('content'))
            write_zip_file.write(
                temp_image.name,
                arcname='word/media/{}'.format(image.get('name')))

    with open(temp_doc.name, 'rb') as read_file:
        return read_file.read()



import os

from django.test import TestCase, RequestFactory

from test_template_engines.fake_app.models import Bidon
from test_template_engines.fake_app.views import OdtTemplateView, DocxTemplateView

from .settings import TEMPLATES_PATH


class TestOdtTemplateView(TestCase):

    def setUp(self):
        self.factory = RequestFactory()
        self.object = Bidon.objects.create(name='Michel')
        self.request = self.factory.get('')

    def test_view_works(self):
        OdtTemplateView.template_name = os.path.join(TEMPLATES_PATH, 'works.odt')
        response = OdtTemplateView.as_view()(self.request, **{'pk': self.object.pk}).render()
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.content), 27160)

    def test_view_works_with_new_line(self):
        OdtTemplateView.template_name = os.path.join(TEMPLATES_PATH, 'works.odt')
        obj = Bidon.objects.create(name='Michel\nPierre')
        response = OdtTemplateView.as_view()(self.request, **{'pk': obj.pk}).render()
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.content), 27169)

    def test_view_works_with_bold_text(self):
        OdtTemplateView.template_name = os.path.join(TEMPLATES_PATH, 'works.odt')
        obj = Bidon.objects.create(name='Michel <b>Pierre</b>')
        response = OdtTemplateView.as_view()(self.request, **{'pk': obj.pk}).render()
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.content), 27226)

    def test_view_empty_image(self):
        OdtTemplateView.template_name = os.path.join(TEMPLATES_PATH, 'empty_image.odt')
        with self.assertRaises(IOError):
            OdtTemplateView.as_view()(self.request, **{'pk': self.object.pk}).render()

    def test_view_bad_image(self):
        OdtTemplateView.template_name = os.path.join(TEMPLATES_PATH, 'bad_image.odt')
        with self.assertRaises(AttributeError):
            OdtTemplateView.as_view()(self.request, **{'pk': self.object.pk}).render()

    def test_bad_image_content(self):
        OdtTemplateView.template_name = os.path.join(TEMPLATES_PATH, 'bad_content_image.odt')
        with self.assertRaises(TypeError):
            OdtTemplateView.as_view()(self.request, **{'pk': self.object.pk}).render()

    def test_view_resize(self):
        OdtTemplateView.template_name = os.path.join(TEMPLATES_PATH, 'resize.odt')
        response = OdtTemplateView.as_view()(self.request, **{'pk': self.object.pk}).render()
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.content), 27137)


class TestDocxTemplateView(TestCase):

    def setUp(self):
        self.factory = RequestFactory()
        self.object = Bidon.objects.create(name='Michel')
        self.request = self.factory.get('')

    def test_view_works(self):
        DocxTemplateView.template_name = os.path.join(TEMPLATES_PATH, 'works.docx')
        response = DocxTemplateView.as_view()(self.request, **{'pk': self.object.pk}).render()
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.content), 41742)

    def test_view_works_with_new_line(self):
        DocxTemplateView.template_name = os.path.join(TEMPLATES_PATH, 'works.docx')
        obj = Bidon.objects.create(name='Michel\nPierre')
        response = DocxTemplateView.as_view()(self.request, **{'pk': obj.pk}).render()
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.content), 41753)

    def test_view_works_with_bold_text(self):
        DocxTemplateView.template_name = os.path.join(TEMPLATES_PATH, 'works.docx')
        obj = Bidon.objects.create(name='Michel <b>Pierre</b>')
        response = DocxTemplateView.as_view()(self.request, **{'pk': obj.pk}).render()
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.content), 41775)

    def test_view_empty_image(self):
        DocxTemplateView.template_name = os.path.join(TEMPLATES_PATH, 'empty_image.docx')
        with self.assertRaises(IOError):
            DocxTemplateView.as_view()(self.request, **{'pk': self.object.pk}).render()

    def test_view_resize(self):
        DocxTemplateView.template_name = os.path.join(TEMPLATES_PATH, 'resize.docx')
        response = DocxTemplateView.as_view()(self.request, **{'pk': self.object.pk}).render()
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.content), 59982)



import re
from io import BytesIO

from PIL import Image

DIM_REGEX = r'^(?P<v>(\d|\.)+)(?P<u>[a-z]*)$'

DOCX_PAGE_WIDTH = 6120130
DOCX_PAGE_HEIGHT = 9251950

ODT_PAGE_WIDTH = 16697
ODT_PAGE_HEIGHT = 28815

CONV_MAPPING = {
    '': lambda v: v,
    'dxa': lambda v: v,
    'pt': lambda v: v * 20,
    'px': lambda v: (v * 0.75) * 20,
    'in': lambda v: v * 72 * 20,
    'cm': lambda v: (v / 2.54) * 72 * 20,
    'emu': lambda v: v / 635,
}


def size_parser(dim, odt=True):
    """
    Convert `dxa`, `pt`, `px`, `in`, `cm`, `emu` to `dxa` if odt else to `emu`.

    :param dim: {value}{unit}
    :type dim: str
    """
    if isinstance(dim, int):
        return dim

    find_dim = re.search(DIM_REGEX, dim)
    value = float(find_dim.group('v'))
    unit = find_dim.group('u')

    if odt:
        return CONV_MAPPING[unit](value)
    else:
        return CONV_MAPPING[unit](value) * 635


def automatically_resize(bimage, odt=True):
    """
    Automatically resize the image to fit the page.

    param bimage: an image.
    :type bimage: bytes

    :param odt: Optional
    :type odt: boolean
    """
    buffer = BytesIO(bimage)
    with Image.open(buffer) as img_reader:
        width, height = img_reader.size
        if odt:
            ratio = min(ODT_PAGE_WIDTH / width, ODT_PAGE_HEIGHT / height)
        else:
            ratio = min(DOCX_PAGE_WIDTH / width, DOCX_PAGE_HEIGHT / height)
        return (width * ratio), (height * ratio)


def resize(bimage, width, height, odt=True):
    """
    Automatically resize the image to fit the page. if no width or height has been specified,
    otherwise convert them.

    :param bimage: an image.
    :type bimage: bytes

    :param width: {value}{unit}
    :type width: str

    :param height: {value}{unit}
    :type height: str

    :param odt: Optional
    :type odt: boolean
    """
    if not width and not height:
        width, height = automatically_resize(bimage, odt=odt)
    else:
        width = size_parser(width, odt=odt)
        height = size_parser(height, odt=odt)

    return width, height



from django import template
from django.utils.safestring import mark_safe

from .utils import resize

register = template.Library()
DOCX_IMAGE = (
    '</w:t>'
    + '</w:r>'
    + '</w:p>'
    + '<w:p>'
    + '<w:r>'
    + '<w:drawing>'
    + '<wp:anchor behindDoc="0" distT="0" distB="0" distL="0" distR="0" simplePos="0" locked="0"'
    + ' layoutInCell="1" allowOverlap="1" relativeHeight="2">'
    + '<a:graphic xmlns:a="http://schemas.openxmlformats.org/drawingml/2006/main">'
    + '<a:graphicData uri="http://schemas.openxmlformats.org/drawingml/2006/picture">'
    + '<pic:pic xmlns:pic="http://schemas.openxmlformats.org/drawingml/2006/picture">'
    + '<pic:blipFill>'
    + '<a:blip r:embed="{0}">'
    + '</a:blip>'
    + '<a:stretch>'
    + '<a:fillRect/>'
    + '</a:stretch>'
    + '</pic:blipFill>'
    + '<pic:spPr bwMode="auto">'
    + '<a:xfrm>'
    + '<a:off x="0" y="0"/>'
    + '<a:ext cx="{1}" cy="{2}"/>'
    + '</a:xfrm>'
    + '<a:prstGeom prst="rect">'
    + '<a:avLst/>'
    + '</a:prstGeom>'
    + '</pic:spPr>'
    + '</pic:pic>'
    + '</a:graphicData>'
    + '</a:graphic>'
    + '</wp:anchor>'
    + '</w:drawing>'
    + '</w:r>'
    + '</w:p>'
    + '<w:p>'
    + '<w:r>'
    + '<w:t>'
)


@register.simple_tag
def docx_image_loader(image):
    """
    Replace a tag by an image you specified.
    You must add an entry to the ``context`` var that is a dict with ``'images'`` as key and other
    dicts in it with at least a ``content`` key whose value is a byte object and a ``name`` key.
    You can also specify ``width`` and ``height``,
    otherwise it will automatically resize your image.
    """
    name = image.get('name')
    width = image.get('width')
    height = image.get('height')
    content = image.get('content')

    width, height = resize(content, width, height, odt=False)

    return mark_safe(DOCX_IMAGE.format(name, width, height))  # nosec



import re
from zipfile import ZipFile

from django.conf import settings
from django.template import Context
from django.template.context import make_context

from . import NEW_LINE_TAG, BOLD_START_TAG, BOLD_STOP_TAG
from .abstract import AbstractEngine, AbstractTemplate
from .utils import modify_libreoffice_doc, add_image_in_docx_template


class DocxTemplate(AbstractTemplate):
    """
    Handles docx templates.
    """

    def __init__(self, template, template_path=None):
        """
        :param template: the template to fill.
        :type template: django.template.Template

        :param template_path: path to the template.
        :type template_path: str
        """
        super().__init__(template)
        self.template_path = template_path

    def clean_bold_tag(self, data):
        nb_bold_tag = len(re.findall(BOLD_START_TAG, data))
        if nb_bold_tag > 0:
            for _ in range(nb_bold_tag):
                data = re.sub(
                    (
                        '<w:r><w:rPr>((<w:[^>]+>)*)</w:rPr><w:t>([^<]*){0}([^<]*){1}'
                        .format(BOLD_START_TAG, BOLD_STOP_TAG)),
                    (
                        '<w:r>'
                        + '<w:rPr>'
                        + '\\g<1>'
                        + '</w:rPr>'
                        + '<w:t>'
                        + '\\g<3>'
                        + '</w:t>'
                        + '</w:r>'
                        + '<w:r>'
                        + '<w:rPr>'
                        + '\\g<1>'
                        + '<w:b w:val="true"/>'
                        + '</w:rPr>'
                        + '<w:t>'
                        + '&#xA0;'
                        + '\\g<4>'
                        + '&#xA0;'
                        + '</w:t>'
                        + '</w:r>'
                        + '<w:r>'
                        + '<w:rPr>'
                        + '\\g<1>'
                        + '</w:rPr>'
                        + '<w:t>'
                    ),
                    data,
                )
        return data

    def clean_new_lines(self, data):
        nb_nl = len(re.findall(NEW_LINE_TAG, data))
        for _ in range(nb_nl):
            data = re.sub(
                '<w:t>([^<{0}]*){0}'.format(NEW_LINE_TAG),
                '<w:t>\\g<1></w:t><w:br/><w:t>',
                data,
            )
        return data

    def render(self, context=None, request=None):
        """
        Fills a docx template with the context obtained by combining the `context` and` request`
        parameters and returns a docx file as a byte object.
        """
        context = make_context(context, request)
        rendered = self.template.render(Context(context))
        rendered = self.clean(rendered)
        docx_content = modify_libreoffice_doc(self.template_path, 'word/document.xml', rendered)
        for _, image in context.get('images', {}).items():
            docx_content = add_image_in_docx_template(docx_content, image)
        return docx_content


class DocxEngine(AbstractEngine):
    """
    Docx template engine.

    ``app_dirname`` is equal to 'templates' but you can change this value by adding
    an ``DOCX_ENGINE_APP_DIRNAME`` setting in your settings.
    By default, ``sub_dirname`` is equal to 'docx' but you can change this value by adding
    an ``DOCX_ENGINE_SUB_DIRNAME`` setting in your settings.
    By default, ``DocxTemplate`` is used as template_class.
    """
    sub_dirname = getattr(settings, 'DOCX_ENGINE_SUB_DIRNAME', 'docx')
    app_dirname = getattr(settings, 'DOCX_ENGINE_APP_DIRNAME', 'templates')
    template_class = DocxTemplate
    mime_type = 'application/vnd.openxmlformats-officedocument.wordprocessingml.document'

    def get_template_content(self, template_path):
        """
        Returns the contents of a template before modification, as a string.
        """
        with ZipFile(template_path, 'r') as zip_file:
            b_content = zip_file.read('word/document.xml')
        return b_content.decode()

    def get_template(self, template_name):
        template_path = self.get_template_path(template_name)
        content = self.get_template_content(template_path)
        return self.from_string(content, template_path=template_path)



from os.path import abspath, dirname, join

ROOT = dirname(dirname(dirname(abspath(__file__))))
TESTS_PATH = join(ROOT, 'template_engines', 'tests')
STATIC_PATH = join(ROOT, 'test_template_engines', 'static')
TEMPLATES_PATH = join(STATIC_PATH, 'templates')
SCREENSHOTS_PATH = join(STATIC_PATH, 'screenshots')

ODT_TEMPLATE_PATH = join(TEMPLATES_PATH, 'odt', 'template.odt')
DOCX_TEMPLATE_PATH = join(TEMPLATES_PATH, 'docx', 'template.docx')

CONTENT_SCREENSHOT_PATH = join(SCREENSHOTS_PATH, 'content_screenshot.txt')
DOCX_CONTENT_SCREENSHOT = join(SCREENSHOTS_PATH, 'docx_content_screenshot.txt')
RENDERED_CONTENT_SCREENSHOT = join(SCREENSHOTS_PATH, 'rendered_content_screenshot.txt')
DOCX_RENDERED_CONTENT_SCREENSHOT = join(SCREENSHOTS_PATH, 'docx_rendered_content_screenshot.txt')

IMAGE_PATH = join(STATIC_PATH, 'makina-corpus.png')



from io import BytesIO
from zipfile import ZipFile

from django.template import Template
from django.template.exceptions import TemplateDoesNotExist
from django.test import TestCase

from template_engines.backends.odt import OdtEngine, OdtTemplate
from ..settings import (
    CONTENT_SCREENSHOT_PATH, DOCX_TEMPLATE_PATH, ODT_TEMPLATE_PATH,
    RENDERED_CONTENT_SCREENSHOT, TEMPLATES_PATH)


class TestOdtEngine(TestCase):

    def setUp(self):
        self.params = {
            'NAME': 'odt',
            'DIRS': [TEMPLATES_PATH],
            'APP_DIRS': False,
            'OPTIONS': [],
        }
        self.odt_engine = OdtEngine(self.params)

    def test_get_template_path_works(self):
        self.assertEqual(self.odt_engine.get_template_path(ODT_TEMPLATE_PATH), ODT_TEMPLATE_PATH)

        params_no_specified_dirs_no_app_dirs = {
            'NAME': 'odt',
            'DIRS': [],
            'APP_DIRS': False,
            'OPTIONS': [],
        }
        odt_engine_no_specified_dirs_no_app_dirs = OdtEngine(params_no_specified_dirs_no_app_dirs)
        with self.assertRaises(TemplateDoesNotExist):
            odt_engine_no_specified_dirs_no_app_dirs.get_template_path('template.odt')

        params_dirs_specified = {
            'NAME': 'odt',
            'DIRS': [TEMPLATES_PATH],
            'APP_DIRS': False,
            'OPTIONS': [],
        }
        odt_engine_dirs_specified = OdtEngine(params_dirs_specified)
        self.assertEqual(odt_engine_dirs_specified.get_template_path('template.odt'),
                         ODT_TEMPLATE_PATH)

    def test_get_template_path_bad_template_name(self):
        with self.assertRaises(TemplateDoesNotExist):
            self.odt_engine.get_template_path('bad_name')

    def test_get_template_content_works(self):
        with open(CONTENT_SCREENSHOT_PATH, 'r') as read_file:
            self.assertEqual(self.odt_engine.get_template_content(ODT_TEMPLATE_PATH),
                             read_file.read())

    def test_get_template_works(self):
        template = self.odt_engine.get_template(ODT_TEMPLATE_PATH)
        self.assertIsInstance(template, OdtTemplate)
        self.assertIsInstance(template.template, Template)
        self.assertEqual(template.template_path, ODT_TEMPLATE_PATH)

    def test_bad_template(self):
        with self.assertRaises(TemplateDoesNotExist):
            self.odt_engine.get_template(DOCX_TEMPLATE_PATH)

    def test_render(self):
        class Obj:
            name = 'Michel'
        template = self.odt_engine.get_template('template.odt')
        rendered = template.render(context={'object': Obj()})
        self.assertIsInstance(rendered, bytes)
        buffer = BytesIO(rendered)
        with ZipFile(buffer, 'r') as zip_read_file:
            with open(RENDERED_CONTENT_SCREENSHOT, 'r') as read_file:
                self.assertEqual(zip_read_file.read('content.xml').decode(), read_file.read())



from io import BytesIO
from zipfile import ZipFile

from django.test import TestCase

from template_engines.backends.utils import modify_libreoffice_doc, add_image_in_docx_template
from ..settings import ODT_TEMPLATE_PATH, DOCX_TEMPLATE_PATH, IMAGE_PATH


class TestUtils(TestCase):

    def test_modify_libreoffice_doc_with_odt_works(self):
        new_file = modify_libreoffice_doc(ODT_TEMPLATE_PATH, 'content.xml', '')
        # Checks the return type
        self.assertIsInstance(new_file, bytes)

        buffer = BytesIO(new_file)
        with ZipFile(buffer, 'r') as buffer_zip_obj:
            with ZipFile(ODT_TEMPLATE_PATH, 'r') as odt_zip_obj:
                buffer_filename_list = list(map(lambda e: e.filename, buffer_zip_obj.infolist()))
                odt_filename_list = list(map(lambda e: e.filename, odt_zip_obj.infolist()))
                # Checks that the archive contains the same files
                self.assertEqual(buffer_filename_list, odt_filename_list)

                for filename in odt_filename_list:
                    if filename != 'content.xml':
                        # Checks that the files are dientical
                        self.assertEqual(buffer_zip_obj.read(filename), odt_zip_obj.read(filename))
                    else:
                        self.assertEqual(buffer_zip_obj.read(filename), b'')

    def test_modify_libreoffice_doc_with_docx_works(self):
        new_file = modify_libreoffice_doc(DOCX_TEMPLATE_PATH, 'word/document.xml', '')
        # Checks the return type
        self.assertIsInstance(new_file, bytes)

        buffer = BytesIO(new_file)
        with ZipFile(buffer, 'r') as buffer_zip_obj:
            with ZipFile(DOCX_TEMPLATE_PATH, 'r') as odt_zip_obj:
                buffer_filename_list = list(map(lambda e: e.filename, buffer_zip_obj.infolist()))
                odt_filename_list = list(map(lambda e: e.filename, odt_zip_obj.infolist()))
                # Checks that the archive contains the same files
                self.assertEqual(buffer_filename_list, odt_filename_list)

                for filename in odt_filename_list:
                    if filename != 'word/document.xml':
                        # Checks that the files are dientical
                        self.assertEqual(buffer_zip_obj.read(filename), odt_zip_obj.read(filename))
                    else:
                        self.assertEqual(buffer_zip_obj.read(filename), b'')

    def test_add_image_in_docx_template_works(self):
        img_content = open(IMAGE_PATH, 'rb').read()
        new_file = add_image_in_docx_template(
            open(DOCX_TEMPLATE_PATH, 'rb').read(),
            {
                'name': 'makinacorpus.png',
                'content': img_content,
            })
        self.assertIsInstance(new_file, bytes)
        buffer = BytesIO(new_file)
        with ZipFile(buffer, 'r') as buffer_zip_obj:
            img = buffer_zip_obj.read('word/media/makinacorpus.png')
            self.assertEqual(img_content, img)
            self.assertEqual(
                buffer_zip_obj.read('word/_rels/document.xml.rels'),
                b''.join([
                    b'<?xml version="1.0" encoding="UTF-8"?>',
                    b'\n<Relationships xmlns="http://schemas.openxmlformats.org/package/2006/relationships">',
                    b'<Relationship Id="rId1" ',
                    b'Type="http://schemas.openxmlformats.org/officeDocument/2006/relationships/styles" ',
                    b'Target="styles.xml"/><Relationship Id="rId2" ',
                    b'Type="http://schemas.openxmlformats.org/officeDocument/2006/relationships/fontTable"',
                    b' Target="fontTable.xml"/><Relationship Id="rId3" ',
                    b'Type="http://schemas.openxmlformats.org/officeDocument/2006/relationships/settings"',
                    b' Target="settings.xml"/>\n<Relationship Id="makinacorpus.png" ',
                    b'Type="http://schemas.openxmlformats.org/officeDocument/2006/relationships/image"',
                    b' Target="media/makinacorpus.png"/></Relationships>'
                ])
            )



from glob import glob
from os.path import isdir, isfile, join

from django.template import Template
from django.template.backends.base import BaseEngine
from django.template.loader import TemplateDoesNotExist
from django.utils.functional import cached_property
from magic import from_file


class AbstractTemplate:
    """
    Gives the architecture of a basic template and one implemented method:
    ``clean``.

    ``render`` must be implemented.
    """

    def __init__(self, template):
        self.template = template

    def clean(self, data):
        """
        Method for cleaning rendered data. For this to work, you must implement methods whose names
        start with ``clean_``.
        """
        unit_clean_method_names = list(filter(lambda e: e.startswith('clean_'), dir(self)))
        unit_clean_methods = list(map(lambda e: getattr(self, e), unit_clean_method_names))
        for clean_method in unit_clean_methods:
            data = clean_method(data)
        return data

    def render(self, context=None, request=None):
        """
        Fills a template with the context obtained by combining the `context` and` request`
        parameters and returns a file as a byte object.
        """
        raise NotImplementedError()


class AbstractEngine(BaseEngine):
    """
    Gives the architecture of a basic template engine and two implemented methods:
    ``get_template_path`` and ``from_string``.

    Can be specified:

    * ``app_dirname``, the folder name which contains the templates in application directories,
    * ``sub_dirname``, the folder name of the subdirectory in the templates directory,
    * ``template_class``, your own template class with a ``render`` method.

    ``get_template`` must be implemented.
    """
    app_dirname = None
    sub_dirname = None
    template_class = None
    mime_type = None

    def __init__(self, params):
        params = params.copy()
        self.options = params.pop('OPTIONS')
        super().__init__(params)

    @cached_property
    def template_dirs(self):
        t_dirs = super().template_dirs
        if self.sub_dirname:
            t_dirs += tuple([join(p, self.sub_dirname)
                             for p in t_dirs
                             if isdir(join(p, self.sub_dirname))])
        return t_dirs

    def from_string(self, template_code, **kwargs):
        return self.template_class(Template(template_code), **kwargs)

    def check_mime_type(self, path):
        fmime_type = from_file(path, mime=True)
        if fmime_type != self.mime_type:
            raise TemplateDoesNotExist('Bad template.')

    def get_template_path(self, template_name):
        """
        Check if a template named ``template_name`` can be found in a list of directories. Returns
        the path if the file exists or raises ``TemplateDoesNotExist`` otherwise.
        """
        if isfile(template_name):
            self.check_mime_type(template_name)
            return template_name
        template_path = None
        for directory in self.template_dirs:
            abstract_path = join(directory, template_name)
            path = glob(abstract_path)
            if path:
                template_path = path[0]
                break
        if template_path is None:
            raise TemplateDoesNotExist(f'Unknown: {template_name}')
        self.check_mime_type(template_path)
        return template_path

    def get_template(self, template_name):
        raise NotImplementedError()
